<?php
/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://searchandfilter.com
 * @since      3.0.0
 *
 * @package    Search_Filter
 */

namespace Search_Filter;

use Search_Filter\Core\Scripts;
use Search_Filter\Util;
use Search_Filter\Core\Icons;
use Search_Filter\Fields\Field;
use Search_Filter\Queries\Query;

// If this file is called directly, abort.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Handles all Admin facing functionality
 */
class Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    3.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    3.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	private $screens;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    3.0.0
	 * @param  string $plugin_name The name of this plugin.
	 * @param  string $version     The version of this plugin.
	 * @param  class  $screens     Instance of the screens class.
	 */
	public function __construct( $plugin_name, $version, $screens ) {
		$this->plugin_name = $plugin_name;
		$this->version     = $version;
		$this->screens     = $screens;
	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    3.0.0
	 */
	public function enqueue_styles() {

		if ( ! $this->screens->is_search_filter_screen() ) {
			return;
		}
		Icons::load();

		// TODO - I'm not sure instantiating the frontend class is the best way to do this.
		// I prefer to keep the frontend or script loader class as a singleton.
		$plugin_frontend = new \Search_Filter\Frontend( $this->plugin_name, $this->version );
		$plugin_frontend->enqueue_styles();

		$registered_styles = array(
			$this->plugin_name . '-flatpickr' => array(
				'src'     => Scripts::get_admin_assets_url() . 'css/vendor/flatpickr.' . Util::get_file_ext( 'css' ),
				'deps'    => array(),
				'version' => $this->version,
				'media'   => 'all',
			),
			$this->plugin_name . '-screen'    => array(
				'src'     => Scripts::get_admin_assets_url() . 'css/admin/app.' . Util::get_file_ext( 'css' ),
				'deps'    => array( 'wp-components' ),
				'version' => $this->version,
				'media'   => 'all',
			),
			$this->plugin_name . '-admin'     => array(
				'src'     => Scripts::get_admin_assets_url() . 'css/admin/admin.' . Util::get_file_ext( 'css' ),
				'deps'    => array( 'wp-components', 'wp-block-editor' ),
				'version' => $this->version,
				'media'   => 'all',
			),
		);

		$registered_styles = apply_filters( 'search-filter/admin/register_styles', $registered_styles );

		foreach ( $registered_styles as $handle => $args ) {
			wp_register_style( $handle, $args['src'], $args['deps'], $args['version'], $args['media'] );
		}

		do_action( 'enqueue_block_editor_assets' );

		$enqueued_styles = array();

		foreach ( $registered_styles as $handle => $args ) {
			if ( wp_style_is( $handle, 'registered' ) ) {
				wp_enqueue_style( $handle );
				$enqueued_styles[] = $handle;
			}
		}

		$enqueued_styles = apply_filters( 'search-filter/admin/enqueue_styles', $enqueued_styles );

		foreach ( $enqueued_styles as $handle ) {
			wp_enqueue_style( $handle );
		}
	}


	private function get_preload_paths() {

		// Setup defaults.
		$section = '';
		$context = '';
		$status  = '';
		$orderby = '';
		$order   = '';
		$search  = '';
		$paged   = '';
		$id      = -1;

		$preload_paths     = array(
			'/search-filter/v1/admin/data',
			'/search-filter/v1/admin/pages',
			'/search-filter/v1/integrations',
			'/search-filter/v1/features',
			'/search-filter/v1/admin/styles/default',
			'/search-filter/v1/admin/settings?section=queries',
			'/search-filter/v1/admin/settings?section=fields',
			'/search-filter/v1/admin/settings?section=styles',
			'/search-filter/v1/admin/settings?section=features',
			'/search-filter/v1/admin/settings?section=integrations',
			'/search-filter/v1/admin/screen/options',
			'/search-filter/v1/admin/screen/dashboard',
		);
		$new_preload_paths = array();

		// Parse URL args.
		$rest_args = array();
		if ( isset( $_GET['section'] ) ) {
			$section = sanitize_key( $_GET['section'] );
		}
		if ( isset( $_GET['status'] ) ) {
			$status              = sanitize_key( $_GET['status'] );
			$rest_args['status'] = $status;

		}
		if ( isset( $_GET['orderby'] ) ) {
			$orderby              = sanitize_key( $_GET['orderby'] );
			$rest_args['orderby'] = $orderby;
		}
		if ( isset( $_GET['order'] ) ) {
			$order              = sanitize_key( $_GET['order'] );
			$rest_args['order'] = $order;
		}
		if ( isset( $_GET['context'] ) ) {
			$context              = sanitize_key( $_GET['context'] );
			$rest_args['context'] = $context;
		}
		if ( isset( $_GET['search'] ) ) {
			// Try to match what encodeURIComponent does (JS function) as that is the route we need to match.
			$revert              = array(
				'%21' => '!',
				'%2A' => '*',
				'%27' => "'",
				'%28' => '(',
				'%29' => ')',
			);
			$search              = strtr( rawurlencode( sanitize_text_field( stripslashes_deep( $_GET['search'] ) ) ), $revert );
			$rest_args['search'] = $search;
		}
		if ( isset( $_GET['paged'] ) ) {
			$paged              = sanitize_key( $_GET['paged'] );
			$rest_args['paged'] = $paged;
		}
		if ( isset( $_GET['edit'] ) ) {
			$id = absint( $_GET['edit'] );
		} elseif ( isset( $_GET['new'] ) ) {
			$id = 0;
		}

		// Lets preload the initial set of results for all sections.
		$items_per_page = 10;
		$screen_options = Settings::get_admin_screen_options();

		$sections = array( 'queries', 'fields', 'styles' );
		// Get the items per page setting to add the to api preload path.
		foreach ( $sections as $section_name ) {
			if ( isset( $screen_options[ $section_name ] ) && isset( $screen_options[ $section_name ]['itemsPerPage'] ) ) {
				$items_per_page = absint( $screen_options[ $section_name ]['itemsPerPage'] );
			}
			$preload_paths[] = '/search-filter/v1/records/' . $section_name . '?per_page=' . $items_per_page;
			$preload_paths[] = '/search-filter/v1/records/counts/' . $section_name;
		}
		// The initial dropdown for the query filter on the fields list screen.
		$preload_paths[] = '/search-filter/v1/records/queries?per_page=20&search=';

		if ( 'queries' === $section || 'fields' === $section || 'styles' === $section ) {
			if ( -1 === $id ) {
				// TODO - preload ?search for all screens and ?query_id (for fields screen)
			} elseif ( $id > 0 ) {
				$new_preload_paths = array(
					'/search-filter/v1/records/' . $section . '/' . $id,
					'/search-filter/v1/settings/options/post-types',
					'/search-filter/v1/settings/options/taxonomies',
					
					'/search-filter/v1/settings/options/post-stati',
					'/search-filter/v1/settings/results-url?integrationType=search',
					'/search-filter/v1/settings/options/queries',
					'/search-filter/v1/settings/options/styles',
					'/search-filter/v1/data/post-types',
				);
				if ( 'queries' === $section ) {
					// Then lets preload the connected fields.
					array_push( $new_preload_paths, '/search-filter/v1/settings/options/post-types?queryId=' . absint( $id ) );
					array_push( $new_preload_paths, '/search-filter/v1/settings/options/taxonomies?queryId=' . absint( $id ) );

					$query = Query::find( array( 'id' => $id ) );
					if ( ! is_wp_error( $query ) ) {
						$integration_type			  = $query->get_attribute( 'integrationType' );
						if ( $integration_type === 'single' ) {
							$selected_post_id = absint( $query->get_attribute( 'singleLocation' ) );
							if ( $selected_post_id > 0 ) {
								array_push( $new_preload_paths, '/search-filter/v1/data/post?id=' . absint( $selected_post_id ) );
								array_push( $new_preload_paths, '/search-filter/v1/settings/results-url?integrationType=single&singleLocation=' . absint( $selected_post_id ) );
							}
						} else if ( $integration_type === 'archive' ) {
							$archive_type = $query->get_attribute( 'archiveType' );
							if ( $archive_type === 'post_type' ) {
								$post_type = $query->get_attribute( 'postType' );
								if ( $post_type && $post_type !== '' ) {
									array_push( $new_preload_paths, "/search-filter/v1/settings/results-url?integrationType={$integration_type}&archiveType={$archive_type}&postType={$post_type}" );
								}
							} else if ( $archive_type === 'taxonomy' ) {
								$taxonomy = $query->get_attribute( 'taxonomy' );
								if ( $taxonomy && $taxonomy !== '' ) {
									array_push( $new_preload_paths, "/search-filter/v1/settings/results-url?integrationType={$integration_type}&archiveType={$archive_type}&taxonomy={$taxonomy}" );
								}
							}
						}
					}
				} elseif ( 'fields' === $section ) {
					// TODO - Check if a query is set, and if so, preload the other api requests relating to taxonomies and post types settings.
					// Always preload the default style.
					$default_style_id = Styles::get_default_styles_id();
					array_push( $new_preload_paths, '/search-filter/v1/records/styles/' . $default_style_id );
					// Preload the currently connected style.

					$field = Field::find( array( 'id' => absint( $id ) ), 'record' );
					if ( ! is_wp_error( $field ) ) {
						$field_attributes = $field->get_attributes();
						$styles_id = isset( $field_attributes['stylesId'] ) ? absint( $field_attributes['stylesId'] ) : 0;
						if ( $styles_id !== 0 ) {
							array_push( $new_preload_paths, '/search-filter/v1/records/styles/' . $styles_id );
						}
						// Preload the currently connected query.
						$query_id = isset( $field_attributes['queryId'] ) ? absint( $field_attributes['queryId'] ) : 0;
						if ( $query_id !== 0 ) {
							array_push( $new_preload_paths, '/search-filter/v1/records/queries/' . $query_id );
							array_push( $new_preload_paths, '/search-filter/v1/settings/options/post-types?queryId=' . absint( $query_id ) );
							array_push( $new_preload_paths, '/search-filter/v1/settings/options/taxonomies?queryId=' . absint( $query_id ) );
						}

					
					}
				} elseif ( 'styles' === $section ) {
					array_push( $new_preload_paths, '/search-filter/v1/admin/styles/default' );
				}
			}
		}
		$preload_paths = array_unique( array_merge( $preload_paths, $new_preload_paths ) );
		return $preload_paths;
	}
	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    3.0.0
	 */
	public function enqueue_scripts() {

		if ( ! $this->screens->is_search_filter_screen() ) {
			return;
		}

		// TODO - I'm not sure instantiating the frontend class is the best way to do this.
		// I prefer to keep the frontend or script loader class as a singleton.
		$plugin_frontend = new \Search_Filter\Frontend( $this->plugin_name, $this->version );
		$plugin_frontend->enqueue_scripts();

		$asset = require SEARCH_FILTER_PATH . 'assets/js/admin/app.asset.php';
		// TODO - need to re-implement dynamic file extension.
		$registered_scripts = array(
			'search-filter-admin' => array(
				'src'     => Scripts::get_admin_assets_url() . 'js/admin/app.' . Util::get_file_ext( 'js' ),
				'deps'    => $asset['dependencies'],
				'version' => $asset['version'],
				'footer'  => true,
			),
		);
		$registered_scripts = apply_filters( 'search-filter/admin/register_scripts', $registered_scripts );

		foreach ( $registered_scripts as $handle => $args ) {
			wp_register_script( $handle, $args['src'], $args['deps'], $args['version'], $args['footer'] );
		}

		$enqueued_scripts = array_keys( $registered_scripts );
		$enqueued_scripts = apply_filters( 'search-filter/admin/enqueue_scripts', $enqueued_scripts );

		foreach ( $enqueued_scripts as $handle ) {
			if ( wp_script_is( $handle, 'registered' ) ) {
				wp_enqueue_script( $handle );
			}
		}

		// Setup the JS data for the admin screen.
		$admin_data = array();
		// TODO - load this through our store.
		$admin_data['editorSettings'] = $this->get_editor_settings();
		$admin_data['editor']         = array(
			'previewContainerClasses' => array(),
		);
		$admin_data['editor']         = apply_filters( 'search-filter/admin/editor/settings', $admin_data['editor'] );
		$admin_data['path']           = wp_parse_url( admin_url(), PHP_URL_PATH );

		Scripts::attach_globals(
			'search-filter-admin',
			'admin',
			(object) $admin_data
		);

		$preload_paths = $this->get_preload_paths();
		Scripts::preload_api_requests( $preload_paths );

		// Setup block editor assets.
		$this->setup_block_editor_assets();
	}
	/**
	 * Gets the editor settings for use in our JS editors
	 *
	 * TODO - keep an eye on global styles project as this will all probably change
	 */
	public function get_editor_settings() {

		$editor_settings = array();

		wp_enqueue_script( 'wp-format-library' );
		wp_enqueue_style( 'wp-format-library' );

		// Need this to trigger so we load theme block editor assets.
		// TODO - we have an issue with using this + Woocommerce...
		$custom_settings = array(
			'siteUrl' => \site_url(),
			'styles'  => \get_block_editor_theme_styles(),
		);
		$editor_settings = \get_block_editor_settings( $custom_settings, null );

		// Non GB, lets just support color palette for now.
		if ( empty( $editor_settings['colors'] ) ) {
			$colors = \get_theme_support( 'editor-color-palette' );
			if ( isset( $colors[0] ) ) {
				$editor_settings['colors'] = $colors[0];
			}
		}
		return $editor_settings;
	}

	public function setup_block_editor_assets() {
		// TODO - we need to keep this in sync with `wp-admin/widgets-form-blocks.php`
		$current_screen = get_current_screen();
		$current_screen->is_block_editor( true );

		remove_action( 'enqueue_block_editor_assets', 'wp_enqueue_editor_block_directory_assets' );

		// TODO - We probably don't need this. OR - we should switch over our admin preload to use `block_editor_rest_api_preload`.
		/*
		$block_editor_context = new \WP_Block_Editor_Context( array( 'name' => 'search-filter/admin' ) );
		$preload_paths        = array(
		);
		// block_editor_rest_api_preload( $preload_paths, $block_editor_context );

		// The widgets editor does not support the Block Directory, so don't load any of
		// its assets. This also prevents 'wp-editor' from being enqueued which we
		// cannot load in the widgets screen because many widget scripts rely on `wp.editor`.
		remove_action( 'enqueue_block_editor_assets', 'wp_enqueue_editor_block_directory_assets' );

		// Preload server-registered block schemas.
		wp_add_inline_script(
			'wp-blocks',
			'wp.blocks.unstable__bootstrapServerSideBlockDefinitions(' . wp_json_encode( get_block_editor_server_block_settings() ) . ');'
		);

		wp_add_inline_script(
			'wp-blocks',
			sprintf( 'wp.blocks.setCategories( %s );', wp_json_encode( get_block_categories( $block_editor_context ) ) ),
			'after'
		);*/
	}
	/**
	 * Check for plugin updates.
	 *
	 * @since    3.0.0
	 */
	public function init_settings() {
		if ( ! $this->screens->is_search_filter_screen() ) {
			return;
		}
	}
	public function update_plugin_check() {
		// Setup the updater.
		$edd_updater = new \Search_Filter\Core\Plugin_Updater(
			'https://searchandfilter.com',
			SEARCH_FILTER_BASE_FILE,
			array(
				'version' => $this->version,
				'license' => 'search-filter-extension-free',
				'item_id' => 514539,
				'author'  => 'Search & Filter',
				'beta'    => false,
			)
		);
	}
}
