<?php
/**
 * Settings Management Class
 *
 * @link       https://searchandfilter.com
 * @since      3.0.0
 * @package    Search_Filter
 */

namespace Search_Filter;

use Search_Filter\Settings\Setting;
use Search_Filter\Core\Exception;
use Search_Filter\Core\WP_Data;

/**
 * The file that defines interactions with S&F settings and edit pages
 *
 * @link       https://searchandfilter.com
 * @since      3.0.0
 *
 * @package    Search_Filter
 * @subpackage Search_Filter/includes
 */

// If this file is called directly, abort.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Keep track of all settings loaded, values etc, for specific search forms, and the global settings
 * Provide an API for modifying externally
 */
class Settings {

	/**
	 * Keep track of whether the settings have been registered yet or not
	 *
	 * @var boolean
	 */
	private static $registered_settings = array();

	/**
	 * Keeps track of all settings registered
	 *
	 * @var array
	 */
	private static $settings = array();
	/**
	 * Keeps track of all section registered
	 *
	 * @var array
	 */
	private static $sections = array();

	/**
	 * Keeps track of the data value of each setting
	 *
	 * @var array
	 */
	private static $data = array();

	/**
	 * Keeps track of the order of the settings
	 *
	 * @var array
	 */
	private static $settings_order = array();
	
	/**
	 * Register built in settings
	 *
	 * @since    3.0.0
	 */
	public static function register( $section, $settings, $groups ) {
		if ( ! isset( self::$registered_settings[ $section ] ) ) {
			self::$registered_settings[ $section ] = false;
			self::$settings[ $section ]            = array();
			self::$settings_order[ $section ]      = array();
			self::$data[ $section ]                = array();
			self::$sections[ $section ]            = array();
		}
		if ( self::$registered_settings[ $section ] ) {
			throw new Exception( sprintf( esc_html__( 'Section `%1$s` already registered', 'search-filter' ), esc_html( $section ) ), SEARCH_FILTER_EXCEPTION_SECTION_REGISTERED );
		}

		if ( ! self::$registered_settings[ $section ] ) {

			// If init hook has not run, we're here too early.
			// if ( ! did_action( 'init' ) ) {
			//	throw new Exception( esc_html__( 'You can only register settings after the `init` action has run', 'search-filter' ), SEARCH_FILTER_EXCEPTION_SETTING_TOO_EARLY );
			//}
			do_action( "search-filter/settings/register/{$section}/before", $settings, $groups);
			
			self::process_settings_data( $settings, $section );
			self::process_section_data( $groups, $section );

			do_action( "search-filter/settings/register/{$section}", $settings, $groups );
		}
	}
	
	// TODO - we need to standardise the order of arguments for these functions.
	public static function add_group_to_section( $name, $label, $section, $type = 'settings', $subgroups = array() ) {
		self::$sections[ $section ][] = array(
			'name'      => $name,
			'label'     => $label,
			'type'      => $type,
			'subgroups' => $subgroups,
		);
	}
	/**
	 * Takes the raw settings data and registers it
	 *
	 * @param array  $settings_data    The data to be stored as settings.
	 * @param string $setting_section  The section name to register the settings to.
	 *
	 * @since    3.0.0
	 */
	private static function process_settings_data( $settings_data, $setting_section ) {

		foreach ( $settings_data as $setting_data ) {
			$args = array(
				'setting' => $setting_data,
				'section' => $setting_section,
			);
			self::register_setting( $args );
		}
	}

	/**
	 * Copies over the section data (name/label)
	 *
	 * @param array  $data        The data to be stored as settings.
	 * @param string $section     The section name to register the settings to.
	 *
	 * @since    3.0.0
	 */
	private static function process_section_data( $data, $section ) {
		self::$sections[ $section ] = $data;
	}
	/**
	 * Registers an individual setting
	 *
	 * @param array $args  The config for a setting.
	 *
	 * @since    3.0.0
	 */
	public static function register_setting( $args ) {
		if ( ( ! isset( $args['setting'] ) ) || ( ! isset( $args['section'] ) ) ) {
			throw new Exception( esc_html__( 'You must supply a `setting` and `section` argument', 'search-filter' ), SEARCH_FILTER_EXCEPTION_SETTING_MISSING_ARGUMENTS );
		}

		if ( ! isset( $args['setting']['name'] ) || empty( trim( $args['setting']['name'] ) ) ) {
			throw new Exception( esc_html__( 'Your setting must have a name', 'search-filter' ), SEARCH_FILTER_EXCEPTION_SETTING_INVALID_NAME );
		}

		if ( true === self::$registered_settings ) {
			throw new Exception( esc_html__( 'You can only register settings inside the action `search-filter/settings/register`', 'search-filter' ), SEARCH_FILTER_EXCEPTION_SETTING_NOT_READY );
		}

		$section = $args['section'];
		$name    = $args['setting']['name'];

		if ( ! isset( self::$settings[ $section ] ) ) {
			throw new Exception( sprintf( esc_html__( 'The section `%1$s` does not exist', 'search-filter' ), esc_html( $section ) ), SEARCH_FILTER_EXCEPTION_SETTING_INVALID_SECTION );
		}

		if ( isset( self::$settings[ $section ][ $name ] ) ) {
			throw new Exception( sprintf( esc_html__( 'A setting with the name `%1$s` already exists', 'search-filter' ), esc_html( $name ) ), SEARCH_FILTER_EXCEPTION_SETTING_EXISTS );
		}

		$args['setting'] = apply_filters( "search-filter/settings/setting/{$name}", $args['setting'], $section );

		self::$settings[ $section ][ $name ] = new Setting( $args['setting'] );

		// Now deal with order.
		if ( isset( $args['after'] ) ) {
			// If "after" is set, position it directly after the named setting.
			$after_name = $args['after'];
			// lookup the setting by its property - "name" - to get its position.
			$found_position = array_search( $after_name, self::$settings_order[ $section ], true );

			if ( false !== $found_position ) {
				$insert_position = $found_position + 1;
				// Make sure the insert position is before or at the end of the array.
				if ( ( $insert_position <= count( self::$settings_order[ $section ] ) ) && ( $insert_position >= 0 ) ) {
					// Then splice it in.
					array_splice( self::$settings_order[ $section ], $insert_position, 0, array( $name ) );
				}
			}
		} elseif ( isset( $args['before'] ) ) {
			// TODO.
		} else {
			array_push( self::$settings_order[ $section ], $name );
		}
	}

	/**
	 * Gets a setting by name and section
	 *
	 * @param string $name     The setting name.
	 * @param string $section  The settings section.
	 *
	 * @since    3.0.0
	 */
	public static function get_setting( $name, $section ) {
		// Find the setting by name.
		if ( ! isset( self::$settings[ $section ] ) ) {
			// Translators: The singular name of the settings section that does not exist.
			// TODO - we need to retun an error (WP_Error?), but probably not throw an exception.

			// throw new Exception( 'search_filter_setting_section_not_exists', sprintf( esc_html__( 'The section `%1$s` does not exist', 'search-filter' ), esc_html( $section ) ) );
		}

		if ( isset( self::$settings[ $section ][ $name ] ) ) {
			return self::$settings[ $section ][ $name ];
		}

		return false;
	}

	/**
	 * Updates a setting by name and section
	 *
	 * @param string $name     The setting name.
	 * @param string $section  The settings section.
	 * @param array  $args     The new setting values.
	 *
	 * @since    3.0.0
	 */
	public static function update_setting( $name, $section, $args ) {

		if ( ! isset( self::$settings[ $section ] ) ) {
			// Translators: The singular internal name of the settings section that does not exist.
			throw new Exception( sprintf( esc_html__( 'The section `%1$s` does not exist', 'search-filter' ), esc_html( $section ) ), SEARCH_FILTER_EXCEPTION_SETTING_INVALID_SECTION );
		}

		if ( ! isset( self::$settings[ $section ][ $name ] ) ) {
			// Translators: The singular name of the setting that does not exist.
			throw new Exception( sprintf( esc_html__( 'A setting with the name `%1$s` does not exist', 'search-filter' ), esc_html( $name ) ), SEARCH_FILTER_EXCEPTION_SETTING_NOT_EXIST );
		}

		self::$settings[ $section ][ $name ]->update( $args );

		return true;
	}

	/**
	 * Gets a group of settings by section.
	 *
	 * @param string $section       The settings section.
	 * @param bool   $return_array  Return as numeric array.
	 *
	 * @return mixed  An array of settings
	 *
	 * @since    3.0.0
	 */
	public static function get_by_section( $section, $return_array = false ) {

		if ( ! $return_array ) {
			return self::$settings[ $section ];
		} else {
			$settings_array = array();
			// Then return the settings as array, ordered.
			foreach ( self::$settings_order[ $section ] as $setting_name ) {
				array_push( $settings_array, self::$settings[ $section ][ $setting_name ]->get_array() );
			}
			return $settings_array;
		}
	}
	/**
	 * Gets the settings defaults for a particular section.
	 *
	 * @param string $section
	 * @return array
	 */
	public static function get_section_defaults( $section ) {
		$defaults = array();
		$settings = self::get_by_section( $section );
		foreach ( $settings as $setting ) {
			$defaults[ $setting->get_name() ] = $setting->get_default();
		}
		return $defaults;
	}

	/**
	 * Gets the sections for a particular section.
	 *
	 * @param string $section  The settings section.
	 *
	 * @since    3.0.0
	 */
	public static function get_sections( $section ) {

		if ( ! isset( self::$sections[ $section ] ) ) {
			self::$sections[ $section ] = array();
		}
		return self::$sections[ $section ];
	}

	// TODO - this shouldn't be in here.
	public static function get_admin_screen_options() {
		$screen_options = get_user_meta( get_current_user_id(), 'search_filter_screen_options', true );
		if ( ! $screen_options || $screen_options === '' ) {
			$screen_options = array(
				'fields'                   => array(
					'itemsPerPage' => '10',
					'columns'      => array( 'name', 'type', 'query', 'status', 'date', 'actions' ),
				),
				'queryEditConnectedFields' => array(
					'itemsPerPage' => '10',
					'columns'      => array( 'name', 'type', 'status', 'date', 'actions' ),
				),
				'queries'                  => array(
					'itemsPerPage' => '10',
					'columns'      => array( 'name', 'fields', 'status', 'date', 'actions' ),
				),
				'styles'                   => array(
					'itemsPerPage' => '10',
					// 'columns' => array( 'name', 'status', 'date' ),
				),
			);
		}
		// Ensure 'name' and 'actions' are present in all columns.
		foreach ( $screen_options as $key => $value ) {
			if ( isset( $value['columns'] ) ) {
				if ( ! in_array( 'name', $value['columns'], true ) ) {
					array_unshift( $screen_options[ $key ]['columns'], 'name' );
				}
				if ( ! in_array( 'actions', $value['columns'], true ) ) {
					array_push( $screen_options[ $key ]['columns'], 'actions' );
				}
			}
		}
		return $screen_options;
	}


	/* TODO - this stuff needs to be moved out of settings, some sort of wp-data wrapper */
	public static function get_taxonomies() {

		$args = array(
			'public' => true,
		);

		$output        = 'objects';
		$operator      = 'and';
		$wp_taxonomies = get_taxonomies( $args, $output, $operator );
		return $wp_taxonomies;
	}

	public static function get_taxonomies_w_archive() {

		$args = array(
			// 'public'   => true,
		);

		$output   = 'objects';
		$operator = 'and';

		$wp_taxonomies = get_taxonomies( $args, $output, $operator );

		$taxonomies = array();
		if ( $wp_taxonomies ) {
			foreach ( $wp_taxonomies  as $taxonomy ) {
				// Taxonomies need to be public, and have a query var (if rewrite is disabled) or have rewrites enabled to have an archive.
				if ( ( $taxonomy->public ) && ( ( $taxonomy->query_var ) || ( $taxonomy->rewrite ) ) ) {
					$item          = array();
					$item['value'] = $taxonomy->name;
					$item['label'] = $taxonomy->label . ' (' . $taxonomy->name . ')';
					array_push( $taxonomies, $item );
				}
			}
		}

		return $taxonomies;
	}


	public static function get_public_post_types() {

		$args = array(
			// 'public'              => true,
			// 'publicly_queryable ' => true,
		);

		$wp_post_types = get_post_types( $args, 'objects' );
		$post_types    = array();

		foreach ( $wp_post_types as $post_type ) {

			if ( ( ( $post_type->has_archive ) && ( $post_type->public ) ) || ( 'post' === $post_type->name ) ) {
				$item          = array();
				$item['value'] = $post_type->name;
				$item['label'] = $post_type->labels->name;
				array_push( $post_types, $item );
			}
		}

		return $post_types;
	}

	public static function get_public_post_type_names() {

		$args = array(
			// 'public'              => true,
			// 'publicly_queryable ' => true,
		);

		$wp_post_types = get_post_types( $args, 'objects' );
		$post_types    = array();

		foreach ( $wp_post_types as $post_type ) {

			if ( ( ( $post_type->has_archive ) && ( $post_type->public ) ) || ( 'post' === $post_type->name ) ) {
				array_push( $post_types, $post_type );
			}
		}

		return $post_types;
	}

	public static function get_post_types() {
		$post_types = WP_Data::get_post_types();

		// $post_types = get_post_types( $args, 'objects' );

		$exclude_post_types = array( 'search-filter', 'revision', 'nav_menu_item', 'shop_webhook' );
		$json_post_types    = array();

		foreach ( $post_types as $post_type ) {

			if ( ! in_array( $post_type->name, $exclude_post_types, true ) ) {
				$item          = array();
				$item['value'] = $post_type->name;
				$item['label'] = $post_type->labels->name;
				array_push( $json_post_types, $item );
			}
		}
		return $json_post_types;
	}

	public static function get_post_stati() {

		$post_stati_objects = get_post_stati( array(), 'objects' );
		$post_stati_ignore  = array( 'auto-draft', 'inherit' );

		$post_stati = array();

		foreach ( $post_stati_objects as $post_status_key => $post_status ) {

			// Don't add any from the ignore list.
			if ( ! in_array( $post_status_key, $post_stati_ignore, true ) ) {

				$post_status = array(
					'value' => $post_status_key,
					'label' => $post_status->label,
				);

				array_push( $post_stati, $post_status );
			}
		}

		return $post_stati;
	}


	/**
	 * Gets all the Query posts
	 *
	 * @return array
	 */
	public static function get_queries_data() {

		/*
		if ( ! empty( self::$queries ) ) {
			return self::$queries;
		}*/

		$sf_query     = new \WP_Query( 'post_type=sf-query&post_status=publish&posts_per_page=-1&fields=ids&suppress_filters=1' );
		$sf_query_ids = $sf_query->get_posts();
		$sf_queries   = array();

		foreach ( $sf_query_ids as $query_id ) {
			// now store their query settings somewhere to use.

			$query_integration = get_post_meta( $query_id, 'search-filter-integration', true );
			$query_settings    = get_post_meta( $query_id, 'search-filter-query', true );

			array_push(
				$sf_queries,
				array(
					'id'                => $query_id,
					'query_integration' => $query_integration,
					'query_settings'    => $query_settings,
				)
			);
		}

		$queries = $sf_queries;
		return $queries;
	}

	// TODO - this is a duplicate function.
	public static function get_post_types_w_archive() {

		$args = array(
			// 'public'                  => true,
			// 'publicly_queryable '     => true
		);

		$wp_post_types = get_post_types( $args, 'objects' );
		$post_types    = array();

		foreach ( $wp_post_types as $post_type ) {

			if ( ( ( $post_type->has_archive ) && ( $post_type->public ) ) || ( 'post' === $post_type->name ) ) {
				$item          = array();
				$item['value'] = $post_type->name;
				$item['label'] = $post_type->labels->name;
				array_push( $post_types, $item );
			}
		}

		return $post_types;
	}
}
