<?php
/**
 * Handles styles presets
 *
 * @link       http://searchandfilter.com
 * @since      3.0.0
 *
 * @package    Search_Filter
 */

namespace Search_Filter;

use Search_Filter\Core\CSS_Loader;
use Search_Filter\Settings;
use Search_Filter\Database\Queries\Style_Presets as Style_Presets_Query;
use Search_Filter\Styles\Style;
use Search_Filter\Styles\Settings as Styles_Settings;

// If this file is called directly, abort.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}
/**
 * A controller for managing all thigns to do with styles presets
 */
class Styles {

	/**
	 * Gets all styles records.
	 */
	public static function get() {
		return self::find( array( 'number' => 0 ) );
	}
	/**
	 * Find styles presets.
	 *
	 * @return array The list of styles presets.
	 */
	public static function find( $args = array() ) {
		// Grab db instance.
		$query_args = array(
			'number'        => 10,
			'no_found_rows' => false, // We want the total, for pagination.
		);
		$query_args = wp_parse_args( $args, $query_args );
		$query      = new Style_Presets_Query( $query_args );
		$records    = array();
		if ( $query ) {
			foreach ( $query->items as $record ) {
				$query_record = array(
					'id'            => $record->get_id(),
					'name'          => $record->get_name(),
					'status'        => $record->get_status(),
					'css'           => $record->get_css(),
					'attributes'    => $record->get_attributes(),
					'date_created'  => $record->get_date_created(),
					'date_modified' => $record->get_date_modified(),
				);
				$records[]    = $query_record;
			}
		}
		return $records;
	}

	/**
	 * Finds the count of styles presets.
	 *
	 * @param [type] $args
	 * @return void
	 */
	public static function find_count( $args ) {
		// Grab db instance.
		$query_args = array(
			'number' => 0,
			'count'  => true,
		);
		$query_args = wp_parse_args( $args, $query_args );
		$query      = new Style_Presets_Query( $query_args );
		return $query->found_items;
	}

	/**
	 * Updates a styles preset in the database.
	 *
	 * @param int   $id The ID of the styles preset to update.
	 * @param array $record The data to update.
	 */
	public static function update( $id, $record ) {
		$query = new Style_Presets_Query(
			array(
				'number' => 1,
				'id'     => $id,
				'fields' => 'ids',
			)
		);

		// If there is a result then we can update.
		if ( ! empty( $query->items ) ) {
			$result = $query->update_item( $id, $record );
		} else {
			// Something went wrong, we're trying to update an ID that doesn't exist.
			$action = 'error';
		}
	}

	/**
	 * Initialise styles.
	 *
	 * Add default styles if none exist, attach to the CSS_Loader.
	 *
	 * @return void
	 */
	public static function init() {

		// TODO - register styles with the CSS_Loader.
		CSS_Loader::register_handler( 'styles', 'Search_Filter\\Styles::get_css' );

		$styles = self::find(
			array(
				'number'        => 1,
				'no_found_rows' => true,
				'status'        => array( 'enabled' ),
			)
		);
		if ( count( $styles ) === 0 ) {
			self::add_default_styles();
		}
		add_action( 'search-filter/record/save', 'Search_Filter\\Styles::save_css', 10, 2 );
		
		// Register settings.
		add_action( 'init', array( __CLASS__, 'register_settings' ), 1 );
	}
	public static function register_settings() {
		// Register settings.
		Settings::register( 'styles', Styles_Settings::get_data(), Styles_Settings::get_groups() );
	}
	/**
	 * Adds a default styles preset to the database.
	 *
	 * @return void
	 */
	public static function add_default_styles() {
		$default_styles = array(
			'name'       => __( 'New Styles', 'search-filter' ),
			'status'     => 'enabled',
			'attributes' => wp_json_encode( (object) self::get_default_attributes() ),
		);

		$new_style = new Style();
		$new_style->set_name( $default_styles['name'] );
		$new_style->set_status( $default_styles['status'] );
		// $new_style->set_css( $default_styles['css'] );
		$new_style->set_attributes( $default_styles['attributes'] );
		$new_style->save();
		$new_id = $new_style->get_id();
		self::set_default_styles_id( $new_id );

		self::save_css( $new_id, 'style' );
	}
	public static function get_default_attributes() {
		// TODO - styles settings should be moved out of field.
		$styles_settings = Styles_Settings::get_data();
		$attributes      = array();
		// Loop through the settings object.
		foreach ( $styles_settings as $style_setting ) {
			$attributes[ $style_setting['name'] ] = $style_setting['default'];
		}
		return $attributes;
	}
	public static function get_default_styles_id() {
		return get_option( 'search_filter_default_styles', 0 );
	}
	public static function set_default_styles_id( $id ) {
		update_option( 'search_filter_default_styles', $id, false );
	}

	/**
	 * When saving a style preset, rebuild the CSS file.
	 *
	 * @param int $id The ID of the styles preset to save.
	 * @param int $section The section of the styles preset to save.
	 */
	public static function save_css( $id, $section ) {
		if ( $section === 'style' ) {
			CSS_Loader::save_css();
		}
	}

	/**
	 * Loop through styles presets, and build their CSS.
	 *
	 * @param int $id The ID of the styles preset to save.
	 */
	public static function get_css() {
		$css = '';
		// Loop through fields, and build their CSS.
		$all_records = self::find(
			array(
				'number' => 0,
				'status' => 'enabled',
			)
		);

		if ( count( $all_records ) > 0 ) {
			foreach ( $all_records as $record ) {
				$css .= self::get_record_css( $record ) . "\r\n";
			}
		}
		return $css;
	}
	/**
	 * Generates or fetches the CSS for a single style preset.
	 *
	 * @since   3.0.0
	 *
	 * @param array $style_preset Associative array of style group data.
	 * @return string The generated CSS.
	 */
	public static function get_record_css( $record ) {
		$css  = '';
		$name = $record['name'];
		// Use cached version.
		$cached_css = $record['css'];
		if ( $cached_css !== '' ) {
			$css .= '/* Style group: ' . esc_html( $name ) . " */\r\n";
			$css .= CSS_Loader::clean_css( $cached_css );
		}

		return $css;
	}

}
