<?php
/**
 * Class for handling the creation of fields.
 *
 * @link       http://searchandfilter.com
 * @since      3.0.0
 *
 * @package    Search_Filter
 * @subpackage Search_Filter/public
 */

namespace Search_Filter\Fields;

use Search_Filter\Core\Exception;

// If this file is called directly, abort.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Instantiates fields using the correct class.
 */
class Field_Factory {
	/**
	 * Keeps track of whether `init` function has been called already.
	 *
	 * @var boolean
	 */
	private static $has_registered_types = false;
	/**
	 * The field classes mapped by their types and their input types.
	 *
	 * @var array
	 */
	private static $fields = array(
		'search'  => array(
			'text' => '\Search_Filter\Fields\Search\Text',
		),
		'filter'  => array(
			'select'      => '\Search_Filter\Fields\Filter\Select',
			'radio'       => '\Search_Filter\Fields\Filter\Radio',
			'checkbox'    => '\Search_Filter\Fields\Filter\Checkbox',
			'link'        => '\Search_Filter\Fields\Filter\Link',
			'button'      => '\Search_Filter\Fields\Filter\Button',
			'date_picker' => '\Search_Filter\Fields\Filter\Date_Picker',
			'slider'      => '\Search_Filter\Fields\Filter\Slider',
		),
		'control' => array(
			'submit'     => '\Search_Filter\Fields\Control\Submit',
			'reset'      => '\Search_Filter\Fields\Control\Reset',
			'sort'       => '\Search_Filter\Fields\Control\Sort',
			'selection'  => '\Search_Filter\Fields\Control\Sort',
			'pagination' => '\Search_Filter\Fields\Control\Pagination',
		),
	);
	/**
	 * Initialises mapping of classes for our different field + input types
	 *
	 * @return void
	 */
	public static function register_types() {
		if ( ! self::$has_registered_types ) {
			do_action( 'search-filter/fields/register' );
			self::$has_registered_types = true;
		}
	}
	/**
	 * Sets has registered field
	 *
	 * Necessary for testing as we can't assert in hooks so must be disabled manually.
	 *
	 * @param bool $has_registered_types Whether fields have been registered.
	 */
	public static function set_has_registered_types( $has_registered_types ) {
		self::$has_registered_types = $has_registered_types;
	}
	/**
	 * Registers a class to be used for a field type and sub input type
	 *
	 * @param string $field_type  The main field type - eg, search, filter or control.
	 * @param string $input_type  The input type for a field type.
	 * @param string $class       The string name of the class.
	 * @return bool
	 * @throws Exception For various conditions, such as invalid field type or input type.
	 */
	public static function register_field_input( $field_type, $input_type, $class ) {

		if ( true === self::$has_registered_types ) {
			throw new Exception( esc_html__( 'You can only register fields inside the action "search-filter/fields/register".', 'search-filter' ), SEARCH_FILTER_EXCEPTION_FIELD_NOT_READY );
		}

		if ( ! isset( self::$fields[ $field_type ] ) ) {
			/* translators: %s is the internal field type - eg search/filter/control */
			throw new Exception( sprintf( esc_html__( 'The field type "%1$s" has not been found.', 'search-filter' ), esc_html( $field_type ) ), SEARCH_FILTER_EXCEPTION_FIELD_INVALID_TYPE );
		}

		if ( isset( self::$fields[ $field_type ][ $input_type ] ) ) {
			/* translators: %1$s is the internal field type - eg search/filter/control', %2$s is the internal input type - eg radio/checkbox/text */
			throw new Exception( sprintf( esc_html__( 'The input type "%1$s" has already been registered for the "%2$s" field type.', 'search-filter' ), esc_html( $input_type ), esc_html( $field_type ) ), SEARCH_FILTER_EXCEPTION_FIELD_EXISTS );
		}

		// Make sure it exists.
		if ( ! class_exists( $class ) ) {
			/* translators: %s is the PHP class name */
			throw new Exception( sprintf( esc_html__( 'The class `%1$s` cannot be found.', 'search-filter' ), esc_html( $class ) ), SEARCH_FILTER_EXCEPTION_FIELD_CLASS_MISSING );
		}

		self::$fields[ $field_type ][ $input_type ] = $class;
		return true;
	}
	/**
	 * Replaces a field class with the one passed
	 *
	 * @param string $field_type  The main type - eg, search, filter or control.
	 * @param string $input_type  The input type for a field type.
	 * @param string $class       The string name of the class.
	 * @return bool
	 * @throws Exception If the fields have already been registered.
	 */
	public static function update_field_input( $field_type, $input_type, $class ) {

		if ( true === self::$has_registered_types ) {
			throw new Exception( esc_html__( 'You can only update fields inside the action "search-filter/fields/register".', 'search-filter' ), SEARCH_FILTER_EXCEPTION_FIELD_NOT_READY );
		}
		// TODO - we need to check if the field + input type exist first.
		self::$fields[ $field_type ][ $input_type ] = $class;
		return true;
	}
	/**
	 * Has registered field input.
	 *
	 * @return void
	 */
	public static function has_registered_field_input( $field_type, $input_type ) {
		return isset( self::$fields[ $field_type ][ $input_type ] );
	}

	/**
	 * Gets a field input class by attributes field type and input type.
	 *
	 * @param array $attributes  Field attributes, can vary based on the `type` attribute.
	 * @return string
	 */
	private static function get_field_class( $attributes ) {
		$field_atts = self::get_field_atts( $attributes );
		$field_type = $field_atts['fieldType'];
		$input_type = $field_atts['inputType'];

		if ( ! isset( self::$fields[ $field_type ][ $input_type ] ) ) {
			return false;
		}
		return self::$fields[ $field_type ][ $input_type ];
	}

	public static function get_field_atts( $attributes ) {

		if ( ! isset( $attributes['type'] ) ) {
			throw new Exception( esc_html__( 'The field type has not been set.', 'search-filter' ), SEARCH_FILTER_EXCEPTION_FIELD_INVALID_TYPE );
		}

		$field_type = $attributes['type'];
		if ( ! isset( self::$fields[ $field_type ] ) ) {
			throw new Exception( esc_html__( 'Invalid field type.', 'search-filter' ), SEARCH_FILTER_EXCEPTION_FIELD_INVALID_TYPE );
		}

		if ( ! isset( $attributes['inputType'] ) && ! isset( $attributes['controlType'] ) ) {
			throw new Exception( esc_html__( 'The input or control type has not been set.', 'search-filter' ), SEARCH_FILTER_EXCEPTION_FIELD_INVALID_INPUT_TYPE );
		}

		$input_type = '';
		if ( $field_type === 'control' ) {
			$input_type = $attributes['controlType'];
		} else {
			$input_type = $attributes['inputType'];
		}
		
		if ( ! isset( self::$fields[ $field_type ][ $input_type ] ) ) {
			/* translators: %s is the internal input type - eg radio/checkbox/text */
			throw new Exception( sprintf( esc_html__( 'The input type `%1$s` does not have a class associated with it. Use `register_field_input`.', 'search-filter' ), esc_html( $input_type ) ), SEARCH_FILTER_EXCEPTION_FIELD_NOT_REGISTERED );
		}
		return array(
			'inputType' => $input_type,
			'fieldType' => $field_type,
		);
	}

	private static function class_ready( $class_ref ) {
		// And make sure that it has a render method.
		if ( ! method_exists( $class_ref, 'render' ) ) {
			/* translators: %s is the class name */
			throw new Exception( sprintf( esc_html__( 'The class `%1$s` does not have a public `render()` method.', 'search-filter' ), esc_html( $class_ref ) ), SEARCH_FILTER_EXCEPTION_FIELD_NO_RENDER_METHOD );
		}
		// And make sure that it has a create method.
		if ( ! method_exists( $class_ref, 'create' ) ) {
			/* translators: %s is the class name */
			throw new Exception( sprintf( esc_html__( 'The class `%1$s` does not have a public static `create()` method.', 'search-filter' ), esc_html( $class_ref ) ), SEARCH_FILTER_EXCEPTION_FIELD_NO_CREATE_METHOD );
		}
		return true;
	}
	/**
	 * Creates a field from attributes.
	 *
	 * Figures out which class to instantiate and returns the result.
	 *
	 * @param array $attributes  Field attributes, can vary based on the `type` attribute.
	 * @return class
	 * @throws Exception For various conditions, such as invalid field type or input type in the attributes.
	 */
	public static function create( $attributes, $context = array() ) {

		// Lookup the class associated with the input type.
		$field_class_ref = self::get_field_class( $attributes );

		if ( self::class_ready( $field_class_ref ) ) {
			// Now create an instance of the correct Field class.
			$field = $field_class_ref::create( $attributes, $context );
			return $field;
		}

		return false;
	}

	/**
	 * Creates a field from db record.
	 *
	 * Figures out which class to instantiate and returns the result.
	 *
	 * @param stdClass $item  The database record for the field.
	 * @return class
	 * @throws Exception For multiple conditions, such as invalid field type or input type.
	 */
	public static function create_from_record( $item ) {
		$attributes = $item->get_attributes();
		// Lookup the class associated with the input type.
		$field_class_ref = self::get_field_class( $attributes );

		if ( self::class_ready( $field_class_ref ) ) {
			// Now create an instance of the correct Field class.
			$field = $field_class_ref::create_from_record( $item );
			return $field;
		}

		return false;
	}
	/**
	 * Creates a field from the record ID.
	 *
	 * Figures out which class to instantiate and returns the result.
	 *
	 * @param number $id  The database record for the field.
	 * @return class
	 * @throws Exception For multiple conditions, such as invalid field type or input type.
	 */
	public static function create_from_id( $id ) {
		$field = Field::find( array( 'id' => $id ) );

		if ( is_wp_error( $field ) ) {
			return $field;
		}

		// Lookup the class associated with the input type.
		$field_class_ref = self::get_field_class( $field->get_attributes() );

		if ( self::class_ready( $field_class_ref ) ) {
			// Now create an instance of the correct Field class.
			$field = $field_class_ref::create_from_record( $field->get_record() );
			return $field;
		}
		// TODO - we probably want to throw an exception or error here.
		return false;
	}
}
