<?php
/**
 * WooCommerce Integration Class
 *
 * @link       https://searchandfilter.com
 * @since      3.0.0
 * @package    Search_Filter/Integrations
 */

namespace Search_Filter\Integrations;

use Search_Filter\Core\WP_Data;
use Search_Filter\Fields\Data\Taxonomy_Options;
use Search_Filter\Integrations;
use Search_Filter\Settings;

// If this file is called directly, abort.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * All WooCommerce integration functionality
 * Add options to admin, integrate with frontend queries
 */
class WooCommerce {

	/**
	 * Keeps track of which of our WC data types are actually taxonomies (for displaying the correct input type).
	 */
	private static $taxonomy_options = array();

	private static $active_query_id = 0;

	/**
	 * Init
	 *
	 * @since    3.0.0
	 */
	public static function init() {

		if ( ! class_exists( 'WooCommerce' ) ) {
			return;
		}

		// Don't do anything until all the settings are init:
		add_action( 'search-filter/settings/init', 'Search_Filter\\Integrations\\WooCommerce::setup', 9 );

	}

	public static function setup() {

		if ( ! Integrations::is_enabled( 'woocommerce' ) ) {
			return;
		}

		// Add WC options to the admin UI.
		self::register_query_settings();
		self::register_field_settings();

		add_filter( 'search-filter/queries/query/get_results_data', 'Search_Filter\\Integrations\\WooCommerce::get_results_data', 10, 2 );
		add_filter( 'search-filter/queries/query/get_results_url/override_url', 'Search_Filter\\Integrations\\WooCommerce::get_results_url', 10, 2 );
		add_filter( 'search-filter/rest-api/get_query_post_types', 'Search_Filter\\Integrations\\WooCommerce::get_query_post_types', 10, 2 );
		add_filter( 'search-filter/query/selector/should_attach', 'Search_Filter\\Integrations\\WooCommerce::attach_query', 10, 3 );
		add_filter( 'search-filter/field/choice/options', 'Search_Filter\\Integrations\\WooCommerce::choice_options', 10, 2 );
		add_filter( 'search-filter/integrations/gutenberg/add_attributes', 'Search_Filter\\Integrations\\WooCommerce::add_block_attributes', 10 );
		add_filter( 'search-filter/field/url_name', 'Search_Filter\\Integrations\\WooCommerce::field_url_name', 10, 2 );
		add_filter( 'search-filter/field/wp_query_args', 'Search_Filter\\Integrations\\WooCommerce::wp_query_args', 10, 3 );

		// Filter the Products block.
		add_filter( 'pre_render_block', '\\Search_Filter\\Integrations\\WooCommerce::pre_render_products_block', 10, 2 );

		// Add support WC products shortcode.
		add_filter( 'shortcode_atts_products', 'Search_filter\\Integrations\\WooCommerce::shortcode_attributes', 10, 3 );
		add_action( 'woocommerce_shortcode_after_products_loop', 'Search_Filter\\Integrations\\WooCommerce::finish_shortcode_loop', 21 );
		add_action( 'woocommerce_shortcode_products_loop_no_results', 'Search_Filter\\Integrations\\WooCommerce::finish_shortcode_loop', 21 );
	}

	public static function pre_render_products_block( $should_short, $block ) {

		if ( $block['blockName'] !== 'core/query' ) {
			return $should_short;
		}

		if ( ! isset ( $block['attrs']['namespace'] ) ) {
			return $should_short;
		}

		if ( $block['attrs']['namespace'] !== 'woocommerce/product-query' ) {
			return $should_short;
		}

		\Search_Filter\Integrations\Gutenberg::try_connect_to_query_loop( $block, 'woocommerce/products_query_block', 'woocommerce/product-query' );

		return $should_short;
	}
	/**
	 * On S&F settings register, add a new setting + update others
	 *
	 * @since    3.0.0
	 */
	public static function register_query_settings() {
		self::add_query_integration();
		self::add_single_integrations();
		self::add_results_setting();
		self::modify_results_url();
	}
	public static function register_field_settings() {
		self::add_wc_setting();
		self::modify_data_type();
		self::modify_input_type();
		self::modify_hide_empty();
		self::modify_hierachical();
		self::modify_hierachical_limit_depth();
		self::modify_hierachical_depth();
	}

	/**
	 * Add attributes to the block editor.
	 */
	public static function add_block_attributes( $attributes ) {
		$attributes['filter']['dataWoocommerce'] = array(
			'type'    => 'string',
			'default' => '',
		);
		return $attributes;
	}
	/**
	 * Modify the query args for the WooCommerce data type.
	 *
	 * @since    3.0.0
	 */
	public static function wp_query_args( $query_args, $attributes, $values ) {
		if ( ! isset( $attributes['dataType'] ) ) {
			return $query_args;
		}
		$data_type = $attributes['dataType'];
		if ( $data_type !== 'woocommerce' ) {
			return $query_args;
		}
		$taxonomy_name = '';
		$data_source   = $attributes['dataWoocommerce'];
		if ( $data_source === 'product_cat' ) {
			$taxonomy_name = 'product_cat';
		} elseif ( $data_source === 'product_tag' ) {
			$taxonomy_name = 'product_tag';
		}

		// If data source starts with "attribute:"
		if ( substr( $data_source, 0, 10 ) === 'attribute:' ) {
			$attribute_name = substr( $data_source, 10 );
			$taxonomy_name  = \wc_attribute_taxonomy_name( $attribute_name );
		}

		$query_values = array();

		foreach ( $values as $tax_term ) {
			if ( term_exists( $tax_term, $taxonomy_name ) ) {
				$query_values[] = $tax_term;
			}
		}
		if ( empty( $query_values ) ) {
			return $query_args;
		}

		if ( ! isset( $query_args['tax_query'] ) ) {
			$query_args['tax_query'] = array();
		}

		// TODO - figure out how to handle this in relation to other taxonomies being set
		// in the query already (ie via the loop block).
		$query_args['tax_query']['relation'] = 'AND';

		$compare_type = 'IN';
		if ( isset( $attributes['multipleMatchMethod'] ) ) {
			$compare_type = $attributes['multipleMatchMethod'] === 'all' ? 'AND' : 'IN';
		}
		if ( $compare_type === 'AND' ) {
			$sub_tax_query = array(
				'relation' => 'AND',
			);
			foreach ( $query_values as $value ) {
				$sub_tax_query[] = array(
					'taxonomy' => $taxonomy_name,
					'field'    => 'slug',
					'terms'    => array( $value ),
				);
			}
			$query_args['tax_query'][] = $sub_tax_query;
		} else {
			$query_args['tax_query'][] = array(
				array(
					'taxonomy' => $taxonomy_name,
					'field'    => 'slug',
					'compare'  => 'IN',
					'terms'    => $query_values,
				),
			);
		}
		return $query_args;
	}
	/**
	 * Set the URL name for the field.
	 *
	 * @since    3.0.0
	 */
	public static function field_url_name( $url_name, $attributes ) {

		$data_type = $attributes['dataType'];

		if ( $data_type !== 'woocommerce' ) {
			return $url_name;
		}

		if ( ! isset( $attributes['dataWoocommerce'] ) ) {
			return $url_name;
		}

		$data_source = $attributes['dataWoocommerce'];

		if ( $data_source === 'product_cat' ) {
			return 'product_cat';
		} elseif ( $data_source === 'product_tag' ) {
			return 'product_tag';
		}

		// If data source starts with "attribute:"
		if ( substr( $data_source, 0, 10 ) === 'attribute:' ) {
			$attribute_name = substr( $data_source, 10 );
			$taxonomy_name  = \wc_attribute_taxonomy_name( $attribute_name );
			return $taxonomy_name;
		}

		return $url_name;
	}
	/**
	 * Populate the options in the fields that need them.
	 *
	 * @since    3.0.0
	 */
	public static function choice_options( $options, $attributes ) {
		$data_type = $attributes['dataType'];
		if ( $data_type !== 'woocommerce' ) {
			return $options;
		}

		if ( count( $options ) > 0 ) {
			return $options;
		}
		$terms = array();

		$data_source     = isset( $attributes['dataWoocommerce'] ) ? $attributes['dataWoocommerce'] : '';
		$hide_empty      = isset( $attributes['hideEmpty'] ) && $attributes['hideEmpty'] === 'yes' ? true : false;
		$is_hierarchical = isset( $attributes['taxonomyHierarchical'] ) && $attributes['taxonomyHierarchical'] === 'yes' ? true : false;
		$order_by        = isset( $attributes['taxonomyOrderBy'] ) ? $attributes['taxonomyOrderBy'] : '';
		$order_dir       = isset( $attributes['taxonomyOrderDir'] ) ? strtoupper( $attributes['taxonomyOrderDir'] ) : '';
		$limit_depth     = isset( $attributes['limitTaxonomyDepth'] ) && $attributes['limitTaxonomyDepth'] === 'yes' ? true : false;
		$args            = array(
			'orderby'    => $order_by,
			'order'      => $order_dir,
			'hide_empty' => $hide_empty,
		);

		$data_tax_depth = 0;
		if ( $is_hierarchical && $limit_depth && isset( $attributes['taxonomyDepth'] ) ) {
			$data_tax_depth = (int) $attributes['taxonomyDepth'];
		}

		$data_tax_terms = array(
			'product_cat',
			'product_tag',
		);
		if ( in_array( $data_source, $data_tax_terms, true ) ) {
			$args['taxonomy'] = $data_source;
			$terms            = WP_Data::get_terms( $args );
		} elseif ( substr( $data_source, 0, 10 ) === 'attribute:' ) {
			// If data source starts with "attribute":
			$attribute_name   = substr( $data_source, 10 );
			$taxonomy_name    = \wc_attribute_taxonomy_name( $attribute_name );
			$args['taxonomy'] = $taxonomy_name;
			$terms            = WP_Data::get_terms( $args );
		}

		$taxonomy_options = new Taxonomy_Options();
		$taxonomy_options->init( $terms );
		if ( $is_hierarchical ) {
			if ( $attributes['inputType'] === 'checkbox' || $attributes['inputType'] === 'radio' ) {
				$options = $taxonomy_options->get_hierarchical_term_options( 'nested', $data_tax_depth );
			} else {
				$options = $taxonomy_options->get_hierarchical_term_options( 'flat', $data_tax_depth );
			}
		} else {
			$options = $taxonomy_options->get_term_options();
		}
		return $options;
	}

	/**
	 * Add the WC shop URL as the results link.
	 *
	 * @since    3.0.0
	 */
	public static function get_results_data( $results_data, $params ) {
		$integration_type = $params['integrationType'];
		if ( $integration_type !== 'woocommerce/shop' ) {
			return $results_data;
		}
		$results_data['url'] = \wc_get_page_permalink( 'shop' );
		return $results_data;
	}
	/**
	 * Add the WC shop URL as the results link.
	 *
	 * @since    3.0.0
	 */
	public static function get_results_url( $url, $params ) {
		$integration_type = $params['integrationType'];
		if ( $integration_type !== 'woocommerce/shop' ) {
			return $url;
		}
		return \wc_get_page_permalink( 'shop' );
	}

	/**
	 * Attach the S&F query to the shop.
	 *
	 * @since    3.0.0
	 */
	public static function attach_query( $should_attach, $saved_query, $query ) {
		// If it's already set to attach, don't change it.
		if ( $should_attach ) {
			return $should_attach;
		}
		if ( is_admin() ) {
			return false;
		}
		if ( ! $query->is_main_query() ) {
			return false;
		}

		$attributes       = $saved_query->get_attributes();
		$integration_type = $attributes['integrationType'];
		if ( $integration_type !== 'woocommerce/shop' ) {
			return $should_attach;
		}
		if ( self::is_shop( $query ) ) {
			return true;
		}
		return $should_attach;
	}

	/**
	 * Conditional to check if we are on the shop page.
	 *
	 * Using is_shop on the frontpage throws errors/warning, so lets circumvent that.
	 * - https://github.com/woothemes/woocommerce/issues/10625#issuecomment-204212754
	 *
	 * @param [type] $query
	 * @return boolean
	 */
	private static function is_shop( $query ) {
		$front_page_id        = get_option( 'page_on_front' );
		$current_page_id      = $query->get( 'page_id' );
		$shop_page_id         = apply_filters( 'woocommerce_get_shop_page_id', get_option( 'woocommerce_shop_page_id' ) );
		$is_static_front_page = 'page' === get_option( 'show_on_front' );

		// Warnings are thrown when using pre_get_posts, Woocommerce & the homepage when using the function `is_shop`.
		if ( $is_static_front_page && $front_page_id === $current_page_id ) {
			// its the homepage so use this function to detect shop.
			$is_shop_page = ( $current_page_id === $shop_page_id ) ? true : false;
		} else {
			// is_shop should work fine.
			$is_shop_page = \is_shop();
		}
		return $is_shop_page;
	}
	/**
	 * Restrict the post types if WooCommerce shop is set.
	 *
	 * @since    3.0.0
	 */
	public static function get_query_post_types( $query_post_types, $params ) {

		$integration_type   = $params['integrationType'];
		$single_integration = $params['singleIntegration'];

		if ( $integration_type === 'single' && $single_integration === 'woocommerce/products_query_block' ) {
			$query_post_types = array(
				'disabled' => true,
				'value'    => array( 'product' ),
			);
		} elseif ( $integration_type === 'single' && $single_integration === 'woocommerce/products_shortcode' ) {
			$query_post_types = array(
				'disabled' => true,
				'value'    => array( 'product' ),
			);
		} elseif ( $integration_type === 'woocommerce/shop' ) {
			$query_post_types = array(
				'disabled' => true,
				// 'value'    => array( 'product', 'product_variation' ),
				'value'    => array( 'product' ),
				// 'message'  => __( 'The WooCommerce Shop only supports the "Product" and "Product Variation" post types.', 'search-filter' ),
			);
		}
		return $query_post_types;
	}

	public static function add_results_setting() {
		$setting = array(
			'name'        => 'resultsUrlWoocommerce',
			'type'        => 'info', // TODO - this can't be right.
			'group'       => 'integration',
			'label'       => __( 'Shop link', 'search-filter' ),
			'help'        => __( 'This is your WooCommerce shop URL', 'search-filter' ),
			'loadingText' => __( 'Fetching...', 'search-filter' ),
			'inputType'   => 'Info',
			'store'       => array(
				'route' => '/settings/results-url',
				'args'  => array(
					'integrationType',
					'archiveType',
					'taxonomy',
					'resultsUrlSingle',
				),
			),
			'dependsOn'   => array(
				'relation' => 'AND',
				'rules'    => array(
					array(
						'option'  => 'integrationType',
						'compare' => '=',
						'value'   => 'woocommerce/shop',
					),
				),
			),
		);
		$args    = array(
			'setting' => $setting,
			'section' => 'queries',
		);

		Settings::register_setting( $args );
	}

	/**
	 * Add WC shop as a query integration type.
	 *
	 * @since    3.0.0
	 */
	public static function modify_results_url() {
		// Get the object for the data_type setting so we can grab its options.
		$integration_type_setting = Settings::get_setting( 'integrationType', 'queries' );
		if ( $integration_type_setting ) {
			// Create the option.
			$wc_integration_type_option = array(
				'label' => __( 'WooCommerce Shop', 'search-filter' ),
				'value' => 'woocommerce/shop',
			);
			$integration_type_setting->add_option( $wc_integration_type_option );
		}
	}
	/**
	 * Add WC shop as a query integration type.
	 *
	 * @since    3.0.0
	 */
	public static function add_query_integration() {
		// get the object for the data_type setting so we can grab its options.
		$integration_type_setting = Settings::get_setting( 'integrationType', 'queries' );
		if ( $integration_type_setting ) {
			// create the option.
			$wc_integration_type_option = array(
				'label' => __( 'WooCommerce Shop', 'search-filter' ),
				'value' => 'woocommerce/shop',
			);
			$integration_type_setting->add_option( $wc_integration_type_option );
		}
	}
	public static function add_single_integrations() {
		// get the object for the data_type setting so we can grab its options.
		$integration_type_setting = Settings::get_setting( 'singleIntegration', 'queries' );
		if ( $integration_type_setting ) {
			$wc_integration_type_option = array(
				'label' => __( 'WooCommerce Products block', 'search-filter' ),
				'value' => 'woocommerce/products_query_block',
			);
			$integration_type_setting->add_option( $wc_integration_type_option );

			$wc_integration_type_option = array(
				'label' => __( 'WooCommerce Products Shortcode', 'search-filter' ),
				'value' => 'woocommerce/products_shortcode',
			);
			$integration_type_setting->add_option( $wc_integration_type_option );
		}
	}
	/**
	 * Add WC as a data type
	 *
	 * @since    3.0.0
	 */
	public static function modify_data_type() {
		// get the object for the data_type setting so we can grab its options.
		$data_type_setting = Settings::get_setting( 'dataType', 'fields' );
		if ( $data_type_setting ) {
			// Create the option.
			$wc_data_type_option = array(
				'label'     => __( 'WooCommerce', 'search-filter' ),
				'value'     => 'woocommerce',
				'dependsOn' => array(
					'relation' => 'AND',
					'rules'    => array(
						array(
							'option'  => 'integrationType',
							'compare' => '=',
							'value'   => 'woocommerce/shop',
						),
					),
				),
			);
			// $data_type_setting->add_option( $wc_data_type_option, array( 'after' => 'control' ) );
			$data_type_setting->add_option( $wc_data_type_option );
		}
	}

	/**
	 * Add WC support to input types
	 *
	 * @since    3.0.0
	 */
	public static function modify_input_type() {

		// Get the object for the input_type setting so we can update it.
		$input_type_setting = Settings::get_setting( 'inputType', 'fields' );

		// We want to change the dependsOn conditions of the individual options, and add support for WC.
		if ( $input_type_setting ) {
			// Create a map of input type options and conditions we want to add.
			$data_type_options = self::get_data_type_options();
			// TODO - this should not be all the options... but right now it is.
			$data_type_values = array_map(
				function ( $option ) {
					return $option['value'];
				},
				$data_type_options
			);

			$depends_options = array(
				'input_type_options' => array( 'select', 'radio', 'checkbox', 'button' ),
				'supports'           => $data_type_values,
			);

			foreach ( $depends_options['input_type_options'] as $input_type ) {
				// Lookup the input type option in the choice options.
				$input_type_option = $input_type_setting->get_option( $input_type );
				if ( ! isset( $input_type_option['dependsOn']['rules'] ) ) {
					$input_type_option['dependsOn']['rules'] = array();
				}
				$depends_condition     = array(
					'relation' => 'AND',
					'action'   => 'hide',
					'rules'    => array(
						array(
							'option'  => 'dataType',
							'value'   => 'woocommerce',
							'compare' => '=',
						),
					),
				);
				$depends_sub_condition = array(
					'relation' => 'OR',
					'action'   => 'hide',
					'rules'    => array(),
				);
				foreach ( $depends_options['supports'] as $depends_value ) {
					$condition = array(
						'option'  => 'dataWoocommerce',
						'compare' => '=',
						'value'   => $depends_value,
					);

					// Add the condition.
					array_push( $depends_sub_condition['rules'], $condition );
				}

				array_push( $depends_condition['rules'], $depends_sub_condition );
				array_push( $input_type_option['dependsOn']['rules'][1]['rules'], $depends_condition );

				// Update the option with the new data.
				$input_type_setting->update_option( $input_type, $input_type_option );
			}

			$input_type_setting = Settings::get_setting( 'inputType', 'fields' );

		}
	}

	/**
	 * Allow WC taxonomies to use the `hide_empty` setting
	 *
	 * @since    3.0.0
	 */
	public static function modify_hide_empty() {
		// get the object for the hide_empty setting so we can update it.
		$hide_empty_setting = Settings::get_setting( 'hideEmpty', 'fields' );
		$setting_data       = $hide_empty_setting->get_data();

		if ( ! $hide_empty_setting ) {
			return;
		}

		if ( ! isset( $setting_data['dependsOn'] ) ) {
			return;
		}

		if ( ! isset( $setting_data['dependsOn']['rules'] ) ) {
			return;
		}
		if ( ! isset( $setting_data['dependsOn']['rules'][1] ) ) {
			return;
		}
		// We want to change the dependsOn conditions of the individual options in the setting
		// and add support for WC.

		$previous_rule                         = $setting_data['dependsOn']['rules'][1];
		$depends_condition                     = array(
			'relation' => 'AND',
			'action'   => 'hide',
			'rules'    => array(
				array(
					'option'  => 'dataType',
					'value'   => 'woocommerce',
					'compare' => '=',
				),
				array(
					'relation' => 'OR',
					'action'   => 'hide',
					'rules'    => array(
						array(
							'option'  => 'dataWoocommerce',
							'value'   => 'product_tag',
							'compare' => '=',
						),
						array(
							'option'  => 'dataWoocommerce',
							'value'   => 'product_cat',
							'compare' => '=',
						),
					),
				),
			),
		);
		$setting_data['dependsOn']['rules'][1] = array(
			'relation' => 'OR',
			'action'   => 'hide',
			'rules'    => array_merge( array( $previous_rule ), array( $depends_condition ) ),
		);

		$hide_empty_setting->update( $setting_data );
	}
	/**
	 * Allow WC taxonomies to use the `hierarchical` settings
	 *
	 * @since    3.0.0
	 */
	public static function modify_hierachical() {
		// get the object for the hide_empty setting so we can update it.
		$hide_empty_setting = Settings::get_setting( 'taxonomyHierarchical', 'fields' );
		$setting_data       = $hide_empty_setting->get_data();

		if ( ! $hide_empty_setting ) {
			return;
		}

		if ( ! isset( $setting_data['dependsOn'] ) ) {
			return;
		}

		if ( ! isset( $setting_data['dependsOn']['rules'] ) ) {
			return;
		}
		if ( ! isset( $setting_data['dependsOn']['rules'][0] ) ) {
			return;
		}
		// We want to change the dependsOn conditions of the individual options in the setting
		// and add support for WC.

		$previous_rule                         = $setting_data['dependsOn']['rules'][1];
		$depends_condition                     = array(
			'relation' => 'AND',
			'action'   => 'hide',
			'rules'    => array(
				array(
					'option'  => 'dataType',
					'value'   => 'woocommerce',
					'compare' => '=',
				),
				array(
					'relation' => 'OR',
					'action'   => 'hide',
					'rules'    => array(
						array(
							'option'  => 'dataWoocommerce',
							'value'   => 'product_tag',
							'compare' => '=',
						),
						array(
							'option'  => 'dataWoocommerce',
							'value'   => 'product_cat',
							'compare' => '=',
						),
					),
				),
			),
		);
		$setting_data['dependsOn']['rules'][1] = array(
			'relation' => 'OR',
			'action'   => 'hide',
			'rules'    => array_merge( array( $previous_rule ), array( $depends_condition ) ),
		);

		$hide_empty_setting->update( $setting_data );
	}
	/**
	 * Allow WC taxonomies to use the `hierarchical` settings
	 *
	 * @since    3.0.0
	 */
	public static function modify_hierachical_limit_depth() {
		// get the object for the hide_empty setting so we can update it.
		$limit_depth_setting = Settings::get_setting( 'limitTaxonomyDepth', 'fields' );
		$setting_data        = $limit_depth_setting->get_data();

		if ( ! $limit_depth_setting ) {
			return;
		}

		if ( ! isset( $setting_data['dependsOn'] ) ) {
			return;
		}

		if ( ! isset( $setting_data['dependsOn']['rules'] ) ) {
			return;
		}
		if ( ! isset( $setting_data['dependsOn']['rules'][0] ) ) {
			return;
		}
		// We want to change the dependsOn conditions of the individual options in the setting
		// and add support for WC.

		$previous_rule                         = $setting_data['dependsOn']['rules'][0];
		$depends_condition                     = array(
			'relation' => 'AND',
			'action'   => 'hide',
			'rules'    => array(
				array(
					'option'  => 'dataType',
					'value'   => 'woocommerce',
					'compare' => '=',
				),
				array(
					'relation' => 'OR',
					'action'   => 'hide',
					'rules'    => array(
						array(
							'option'  => 'dataWoocommerce',
							'value'   => 'product_tag',
							'compare' => '=',
						),
						array(
							'option'  => 'dataWoocommerce',
							'value'   => 'product_cat',
							'compare' => '=',
						),
					),
				),
			),
		);
		$setting_data['dependsOn']['rules'][0] = array(
			'relation' => 'OR',
			'action'   => 'hide',
			'rules'    => array_merge( array( $previous_rule ), array( $depends_condition ) ),
		);

		$limit_depth_setting->update( $setting_data );
	}
	/**
	 * Allow WC taxonomies to use the `hierarchical depth` settings
	 *
	 * @since    3.0.0
	 */
	public static function modify_hierachical_depth() {
		// get the object for the hide_empty setting so we can update it.
		$hierarchical_depth_setting = Settings::get_setting( 'taxonomyDepth', 'fields' );
		$setting_data               = $hierarchical_depth_setting->get_data();

		if ( ! $hierarchical_depth_setting ) {
			return;
		}

		if ( ! isset( $setting_data['dependsOn'] ) ) {
			return;
		}

		if ( ! isset( $setting_data['dependsOn']['rules'] ) ) {
			return;
		}
		if ( ! isset( $setting_data['dependsOn']['rules'][0] ) ) {
			return;
		}
		// We want to change the dependsOn conditions of the individual options in the setting
		// and add support for WC.

		$previous_rule                         = $setting_data['dependsOn']['rules'][0];
		$depends_condition                     = array(
			'relation' => 'AND',
			'action'   => 'hide',
			'rules'    => array(
				array(
					'option'  => 'dataType',
					'value'   => 'woocommerce',
					'compare' => '=',
				),
				array(
					'relation' => 'OR',
					'action'   => 'hide',
					'rules'    => array(
						array(
							'option'  => 'dataWoocommerce',
							'value'   => 'product_tag',
							'compare' => '=',
						),
						array(
							'option'  => 'dataWoocommerce',
							'value'   => 'product_cat',
							'compare' => '=',
						),
					),
				),
			),
		);
		$setting_data['dependsOn']['rules'][0] = array(
			'relation' => 'OR',
			'action'   => 'hide',
			'rules'    => array_merge( array( $previous_rule ), array( $depends_condition ) ),
		);

		$hierarchical_depth_setting->update( $setting_data );
	}

	/**
	 * Gets the options for our data type field.
	 *
	 * @return array
	 */
	public static function get_data_type_options() {
		$data_options = array(
			array(
				'label' => __( 'Tags', 'search-filter' ),
				'value' => 'product_tag',
			),
			array(
				'label' => __( 'Categories', 'search-filter' ),
				'value' => 'product_cat',
			),
			/*
			 array(
				'label' => __( 'Price', 'search-filter' ),
				'value' => 'price',
			),*/
			/*
			array(
				'label' => __( 'In Stock', 'search-filter' ),
				'value' => '6',
			),
			*/
		);
		$wc_tax_attributes = \wc_get_attribute_taxonomies();
		foreach ( $wc_tax_attributes as $attribute ) {
			$data_options[] = array(
				'value' => 'attribute:' . $attribute->attribute_name,
				// Translators: %s is the attribute name.
				'label' => sprintf( __( 'Attribute: %1$s', 'search-filter' ), $attribute->attribute_label ),
			);
		}

		self::$taxonomy_options = $data_options;

		return $data_options;
		// TODO, taxonomy_options won't be a copy of data_options once we add different types of options here (such as price).
	}
	/**
	 * Creates a WC setting which is only shown when the WC data_type
	 * is selected
	 *
	 * @since    3.0.0
	 */
	public static function add_wc_setting() {

		$data_options = self::get_data_type_options();

		$setting = array(
			'name'        => 'dataWoocommerce',
			'label'       => __( 'Data Source', 'search-filter' ),
			'description' => __( 'Select a WooCommerce data type', 'search-filter' ),
			'group'       => 'data',
			'type'        => 'string',
			'inputType'   => 'Select',
			'context'     => array( 'admin/field', 'admin/field/filter', 'block/field/filter' ),
			'placeholder' => __( 'Choose WooCommerce source', 'search-filter' ),
			'options'     => $data_options,
			'dependsOn'   => array(
				'relation' => 'AND',
				'rules'    => array(
					array(
						'option'  => 'dataType',
						'compare' => '=',
						'value'   => 'woocommerce',
					),
				),
			),
			'supports'    => array(
				'previewAPI' => true,
			),
		);

		$args = array(
			'setting' => $setting,
			'section' => 'fields',
			'after'   => 'dataType', // Add it right after the `data_type` setting.
		);

		Settings::register_setting( $args );
	}


	public static function shortcode_attributes( $out, $pairs, $atts ) {
		if ( ! isset( $atts['search_filter_query_id'] ) ) {
			return $out;
		}
		if ( self::$active_query_id !== 0 ) {
			return $out;
		}
		// Remove products shortcode caching.
		$out['cache']          = false;
		self::$active_query_id = absint( $atts['search_filter_query_id'] );
		add_filter( 'woocommerce_shortcode_products_query', 'Search_Filter\\Integrations\\WooCommerce::add_query_to_args', 10, 1 );
		return $out;
	}

	public static function add_query_to_args( $query_args ) {
		// Remove products shortcode caching.
		if ( self::$active_query_id === 0 ) {
			return $query_args;
		}
		$query_args['search_filter_query_id'] = self::$active_query_id;
		// Remove products shortcode caching.
		remove_filter( 'woocommerce_shortcode_products_query', 'Search_Filter\\Integrations\\WooCommerce::add_query_to_args', 10, 1 );
		return $query_args;
	}
	public static function get_active_query_id() {
		return self::$active_query_id;
	}
	public static function finish_shortcode_loop() {
		self::$active_query_id = 0;
	}
}
