<?php
/**
 * Figures out, based on saved S&F Queries, which WP Queries to affect, by assigning `sf_query_id` to the
 * appropriate WP Queries
 *
 * @link       http://searchandfilter.com
 * @since      3.0.0
 *
 * @package    Search_Filter
 * @subpackage Search_Filter/public
 */

namespace Search_Filter\Query;

use Search_Filter\Queries;
use Search_Filter\Queries\Query;

// If this file is called directly, abort.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Figures out which query to "Select" or attach to
 */
class Selector {

	/**
	 * Stores a local copy of our queries.
	 *
	 * @var [type]
	 */
	private static $queries = array();

	/**
	 * Register the stylesheets for the public-facing side of the plugin.
	 *
	 * @since    3.0.0
	 */
	public static function init() {
		add_action( 'init', 'Search_Filter\\Query\\Selector::init_queries', 21 );
		add_action( 'pre_get_posts', 'Search_Filter\\Query\\Selector::attach_ids', 10 );
		add_action( 'pre_get_posts', 'Search_Filter\\Query\\Selector::attach_queries', 11 );

	}
	public static function init_queries() {
		self::$queries = Queries::find(
			array(
				'status' => 'enabled',
				'number' => 0,
			)
		);
	}

	/**
	 * Attach S&F queries by ID.
	 *
	 * @param [type] $wp_query
	 * @return void
	 */
	public static function attach_ids( $wp_query ) {
		$search_filter_id = $wp_query->get( 'search_filter_query_id' );
		if ( empty( $search_filter_id ) ) {
			return;
		}
		// TODO - we probably want to re-use the already looked up self::$queries.
		$query = Query::find(
			array(
				'id'     => $search_filter_id,
				'status' => 'enabled',
			)
		);

		if ( is_wp_error( $query ) ) {
			return;
		}

		$wp_query->set( 'search_filter_queries', array( $query ) );
	}
	/**
	 * Based on saved admin queries, check the current query / page to see if we need to
	 * attach an ID.
	 *
	 * @param stdClass $query The instantiated WP_Query object.
	 */
	public static function attach_queries( $query ) {
		// TODO - store the integration settings in seperate columns so we can look them up,
		// rather than looping through all of them on every page load.
		foreach ( self::$queries as $saved_query ) {
			$id            = $saved_query->get_id();
			$attributes    = $saved_query->get_attributes();
			$should_attach = false;

			if ( ! $should_attach ) {
				// Based on the integration type, check if we need to attach to this query.
				if ( self::should_attach_wp_search_query( $attributes, $query ) ) {
					$should_attach = true;
				} elseif ( self::should_attach_archive_query( $attributes, $query ) ) {
					$should_attach = true;
				}
			}

			// Allow for custom integration types.
			$should_attach = apply_filters( 'search-filter/query/selector/should_attach', $should_attach, $saved_query, $query );

			if ( $should_attach ) {
				// TODO - need to wrap this, and throw an error when setting multiple query IDs.
				$query->set( 'search_filter_queries', array( $saved_query ) );
			}
		}
	}

	/**
	 * Detect whether the query is the one used on WP Search Results page (yoursite.com/?s=) and
	 *
	 * @since    3.0.0
	 * @param object $query  The WP Query object.
	 */
	public static function should_attach_wp_search_query( $attributes, $query ) {

		$integration_type = $attributes['integrationType'];

		if ( $integration_type !== 'search' ) {
			return false;
		}

		if ( is_admin() ) {
			return false;
		}

		if ( ! $query->is_main_query() ) {
			return false;
		}

		if ( ! is_search() ) {
			return false;
		}

		if ( is_archive() ) {
			return false;
		}

		if ( ( $query->is_search() ) && ( isset( $query->query['s'] ) ) ) {
			return true;
		}
		return false;
	}

	/**
	 * Detect whether the query is the one used on WP Search Results page (yoursite.com/?s=) and
	 *
	 * @since    3.0.0
	 * @param object $query  The WP Query object.
	 */
	public static function should_attach_archive_query( $attributes, $query ) {

		$integration_type = $attributes['integrationType'];

		if ( $integration_type !== 'archive' ) {
			return false;
		}

		if ( is_admin() ) {
			return false;
		}

		if ( ! $query->is_main_query() ) {
			return false;
		}

		// Now check if we need to attach to a specific taxonomy archive or post type archive.
		$archive_type = $attributes['archiveType'];

		// Check for the special case of the blog first:
		if ( $archive_type === 'post_type' ) {
			$post_type = $attributes['postType'];
			if ( $post_type === 'post' ) {
				// If the reading setting "homepage displays" is set to "posts".
				if ( is_home() ) {
					return true;
				}
			}
		}

		// So its not the blog, so bail if its not an arhive.
		if ( ! $query->is_archive() ) {
			return false;
		}
		if ( $archive_type === 'post_type' ) {
			$post_type = $attributes['postType'];
			if ( $query->is_post_type_archive( $post_type ) ) {
				return true;
			}

			$archive_filter_taxonomies = isset( $attributes['archiveFilterTaxonomies'] ) ? $attributes['archiveFilterTaxonomies'] : 'no';
			
			if ( $archive_filter_taxonomies === 'yes' ) {
				$taxonomies = get_object_taxonomies( $post_type );
				foreach ( $taxonomies as $taxonomy ) {
					if ( $query->is_tax( $taxonomy ) ) {
						return true;
					}
				}
			}
		} elseif ( $archive_type === 'taxonomy' ) {
			$taxonomy = $attributes['taxonomy'];
			if ( $taxonomy === 'category' ) {
				if ( $query->is_category() ) {
					return true;
				}
			} elseif ( $taxonomy === 'post_tag' ) {
				if ( $query->is_tag() ) {
					return true;
				}
			} else if ( $query->is_tax( $taxonomy ) ) {
				return true;
			}
		}
		return false;
	}
}
