<?php
/**
 * Class for handling the frontend display of a field.
 *
 * @link       http://searchandfilter.com
 * @since      3.0.0
 *
 * @package    Search_Filter
 * @subpackage Search_Filter/public
 */

namespace Search_Filter\Styles;

use Search_Filter\Core\CSS_Loader;
use Search_Filter\Record_Base;

// If this file is called directly, abort.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}


/**
 * The base field for all styles.
 */
class Style extends Record_Base {
	/**
	 * The record store name
	 *
	 * @since    3.0.0
	 * @access   protected
	 * @var      int    $id    ID
	 */
	public $record_store = 'style';


	/**
	 * The meta type for the meta table.
	 *
	 * @var string
	 */
	public static $meta_table = 'search_filter_style';

	/**
	 * The full string of the class name of the query class for this section.
	 *
	 * @since    3.0.0
	 * @access   protected
	 * @var      string    $records_class    The string class name.
	 */
	public static $records_class = 'Search_Filter\\Database\\Queries\\Style_Presets';

	/**
	 * The class name to handle interacting with the record stores.
	 *
	 * @since    3.0.0
	 * @access   protected
	 * @var      string    $base_class    ID
	 */
	public static $base_class = 'Search_Filter\\Styles';
	// public static $settings_class = 'Search_Filter\\Settings\\Style';
	/**
	 * Context for the styles preset, eg, "theme"
	 *
	 * @since    3.0.0
	 * @access   protected
	 * @var      string    $name    Name
	 */
	protected $context = '';
	/**
	 * The CSS string for this style preset.
	 *
	 * @var string
	 */
	private $css = '';

	/**
	 * Overridable function for setting default attributes.
	 *
	 * @return array New default attributes.
	 */
	public function get_default_attributes() {
		$defaults = \Search_Filter\Styles\Settings::get_defaults();
		return $defaults;
	}
	/**
	 * Sets the styles context (theme)
	 *
	 * @param string $context The context to set
	 */
	public function set_context( $context ) {
		$this->context = $context;
	}
	/**
	 * Gets the styles context
	 */
	public function get_context() {
		return $this->context;
	}

	/**
	 * Get the css of the style preset.
	 */
	public function get_css() {
		return $this->css;
	}
	/**
	 * Set the css of the style preset.
	 *
	 * @param string $css The css to set.
	 */
	public function set_css( $css ) {
		$this->css = $css;
	}

	/**
	 * Inits the data from a DB record.
	 *
	 * @param [type] $item Database record.
	 */
	public function load_record( $item ) {
		parent::load_record( $item );
		$this->set_context( $item->get_context() );
		$this->set_css( $item->get_css() );
	}

	public function save( $args = array() ) {
		// Update the CSS.

		$has_id = false;

		if ( $this->get_id() ) {
			$has_id = true;
			$this->regenerate_css();
		}
		$extra_args = array(
			'context' => $this->get_context(),
			'css'     => $this->get_css(),
		);
		$record_id  = parent::save( $extra_args );

		if ( ! $has_id && $record_id ) {
			// If we don't have an ID, then we need to update the ID in the Data_Store.
			$this->regenerate_css();
			$extra_args['css'] = $this->get_css();
			parent::save( $extra_args );
		}

		// TODO - are we clearing our Data_Store once the info has been updated?
		// Or shall we update it rather than clear it?
		return $record_id;
	}

	/**
	 * Regenerates the CSS for the style preset.
	 *
	 * @since   3.0.0
	 */
	public function regenerate_css() {
		$this->css = $this->generate_css();
	}

	/**
	 * Generates the CSS for the style preset.
	 *
	 * @return string The generated CSS.
	 */
	private function generate_css() {
		$css = '';
		// Get the base styles class for the ID.
		$styles_class = '.search-filter--style-' . intval( $this->get_id() );

		$css .= $styles_class . '{';
		$css .= CSS_Loader::clean_css( $this->create_attributes_css( $this->get_attributes() ) );
		$css .= '}';

		return $css;
	}

	/**
	 * Parses the attributes into CSS styles (variables).
	 *
	 * @since   3.0.0
	 *
	 * @param array $attributes  The saved style attributes.
	 *
	 * @return string The generated CSS.
	 */
	public function create_attributes_css( $attributes ) {
		$mapped_css_vars = array(
			'--search-filter-label-text-color'             => 'labelTextColor',
			'--search-filter-input-text-color'             => 'textColor',
			'--search-filter-input-placeholder-text-color' => 'textColor',
			'--search-filter-input-bg-color'               => 'backgroundColor',
			'--search-filter-input-border-color'           => 'primaryAccentColor',
			'--search-filter-input-border-hover-color'     => 'secondaryAccentColor',
			'--search-filter-input-border-focused-color'   => 'tertiaryAccentColor',
			'--search-filter-input-border-accent-color'    => 'primaryAccentColor',
			'--search-filter-input-interactive-color'      => 'primaryAccentColor',
			'--search-filter-input-interactive-hover-color' => 'tertiaryAccentColor',
			'--search-filter-input-icon-color'             => 'primaryAccentColor',
			'--search-filter-input-destructive-color'      => 'primaryAccentColor',
			'--search-filter-input-destructive-hover-color' => 'tertiaryAccentColor',
			'--search-filter-checkable-color'              => 'activeBackgroundColor',
			'--search-filter-checkable-inactive-color'     => 'primaryAccentColor',
			'--search-filter-checkable-label-color'        => 'labelTextColor',
			'--search-filter-button-text-color'            => 'textColor',
			'--search-filter-button-bg-color'              => 'backgroundColor',
			'--search-filter-button-text-hover-color'      => 'textColor',
			'--search-filter-button-bg-hover-color'        => 'backgroundColor',
			'--search-filter-button-text-active-color'     => 'activeTextColor',
			'--search-filter-button-bg-active-color'       => 'activeBackgroundColor',
			'--search-filter-button-border-color'          => 'primaryAccentColor',
			'--search-filter-button-border-hover-color'    => 'secondaryAccentColor',
			'--search-filter-button-border-focus-color'    => 'tertiaryAccentColor',
			'--search-filter-dropdown-hover-color'         => 'activeBackgroundColor',
			'--search-filter-dropdown-active-bg-color'     => 'activeBackgroundColor',
			'--search-filter-dropdown-active-text-color'   => 'activeTextColor',
			'--search-filter-dropdown-active-bg-hover-color' => 'activeBackgroundColor',
		);
		$modifiers       = array(
			'--search-filter-input-placeholder-text-color' => 'AA',
			'--search-filter-dropdown-hover-color'         => '77',
			'--search-filter-dropdown-active-bg-hover-color' => 'CC',
		);

		$css = '';
		foreach ( $mapped_css_vars as $css_var => $attribute_key ) {
			if ( isset( $attributes[ $attribute_key ] ) ) {
				// TODO - we need to handle the different color type... not only hex...
				$css .= $css_var . ':' . sanitize_hex_color( $attributes[ $attribute_key ] );
				if ( isset( $modifiers[ $css_var ] ) ) {
					$css .= $modifiers[ $css_var ];
				}
				$css .= ';';
			}
		}

		$css = apply_filters( 'search-filter/styles/style/create_attributes_css', $css, $attributes );
		return $css;
	}

	public function get_attributes() {
		$attributes = parent::get_attributes();
		$attributes = apply_filters( 'search-filter/styles/style/get_attributes', $attributes, $this->get_id() );
		return $attributes;
	}

}
